package com.nisum.splitmerchantpayment.controllers;

import com.nisum.splitmerchantpayment.entity.CartProduct;
import com.nisum.splitmerchantpayment.entity.OrderDao;
import com.nisum.splitmerchantpayment.model.CartDto;
import com.nisum.splitmerchantpayment.model.CustomerOrderDto;
import com.nisum.splitmerchantpayment.model.CustomerPaymentDto;
import com.nisum.splitmerchantpayment.model.ProductToCartDto;
import com.nisum.splitmerchantpayment.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * This is User Controller class we present all different API's
 * and various endpoints
 */
@RestController
public class UserController {
    @Autowired
    private UserService userService;

    /**
     * This method is used to add products to the cart with using the endpoint
     * as post-mapping
     *
     * @param productToCartDto
     * @return String with response message
     */
    @PostMapping(value = "/carts")
    public ResponseEntity<String> addToCart(@RequestBody ProductToCartDto productToCartDto) {
        String message = userService.addToCart(productToCartDto);
        return ResponseEntity.status(HttpStatus.OK).body(message);
    }

    /**
     * This method is used to get the products added in cart
     * by giving a specific customer phone-number as id by using
     * get-mapping endpoint
     *
     * @param customerPhoneNumber
     * @return cartDto
     */
    @GetMapping(value = "/carts")
    public ResponseEntity<CartDto> getCart(@RequestParam String customerPhoneNumber) {
        List<CartProduct> orderedProducts = userService.getCartList(customerPhoneNumber);
        Double amountPayable = orderedProducts
                .stream()
                .map(cartProductDto -> cartProductDto.getQuantity() * cartProductDto.getPrice())
                .reduce(0.00, (init, cost) -> init + cost);
        return ResponseEntity.status(HttpStatus.OK).body(CartDto.builder()
                .cartProducts(orderedProducts).amountPayable(amountPayable).build());
    }

    /**
     * This method is used to place the order which are present in the
     * cart by using post-mapping and providing specific customer's phone-number
     * and total amount
     *
     * @param customerPaymentDto
     * @return order with generation of orderId
     */
    @PostMapping(value = "/orders")
    public ResponseEntity<OrderDao> placeOrder(@RequestBody CustomerPaymentDto customerPaymentDto) {
        return ResponseEntity.status(HttpStatus.OK)
                .body(userService
                        .placeOrder(customerPaymentDto.getPhoneNumber(), customerPaymentDto.getAmount()));
    }

    /**
     * This method is used to give the list of orders according to their
     * specific customer phone-number by using endpoint get-mapping
     *
     * @param customerPhoneNumber
     * @return ordersList
     */
    @GetMapping(value = "/orders")
    public ResponseEntity<List<OrderDao>> ordersByCustomer(@RequestParam String customerPhoneNumber) {
        List<OrderDao> orderDaoList = userService.getOrdersByCustomer(customerPhoneNumber);
        return ResponseEntity.status(HttpStatus.OK).body(orderDaoList);
    }

    /**
     * This method is used to give the order details by providing specific orderId
     *
     * @param customerOrderDto
     * @return order
     */
    @GetMapping(value = "/customerorders")
    public ResponseEntity<OrderDao> ordersBYId(@RequestBody CustomerOrderDto customerOrderDto) {
        OrderDao orderDao = userService.getOrderByCustomerIdAndOrderId(customerOrderDto.getPhoneNumber
                (), customerOrderDto.getOrderId());
        return ResponseEntity.status(HttpStatus.OK).body(orderDao);
    }
}
