package com.nisum.splitmerchantpayment.service;

import com.nisum.splitmerchantpayment.entity.ProductName;
import com.nisum.splitmerchantpayment.exception.BrandNotFoundException;
import com.nisum.splitmerchantpayment.exception.CategoryNotFoundException;
import com.nisum.splitmerchantpayment.exception.OrderListEmptyException;
import com.nisum.splitmerchantpayment.exception.ProductNameNotFoundException;
import com.nisum.splitmerchantpayment.exception.ProductNotAvailableException;
import com.nisum.splitmerchantpayment.model.ProductDto;
import com.nisum.splitmerchantpayment.repository.BrandRepository;
import com.nisum.splitmerchantpayment.repository.CategoryRepository;
import com.nisum.splitmerchantpayment.repository.OrderRepository;
import com.nisum.splitmerchantpayment.repository.ProductNameRepository;
import com.nisum.splitmerchantpayment.repository.ProductRepository;
import com.nisum.splitmerchantpayment.utils.IDGeneratorUtility;
import com.nisum.splitmerchantpayment.entity.Brand;
import com.nisum.splitmerchantpayment.entity.Category;
import com.nisum.splitmerchantpayment.entity.OrderDao;
import com.nisum.splitmerchantpayment.entity.Product;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.Optional;

@Service
public class AdminServiceImpl implements AdminService {

    @Autowired
    private ProductRepository productRepository;

    @Autowired
    private CategoryRepository categoryRepo;

    @Autowired
    private BrandRepository brandRepo;

    @Autowired
    private ProductNameRepository productNameRepo;

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private IDGeneratorUtility idGeneratorUtility;

    @PersistenceContext
    private EntityManager getEntityManager;

    @Override
    public Product addProduct(ProductDto productDto) throws RuntimeException {

        Brand brand = brandRepo.findById(productDto.getBrand())
                .orElseThrow(() -> new BrandNotFoundException("Brand is not found with id : " + productDto.getBrand()));
        Category category = categoryRepo.findById(productDto.getCategory())
                .orElseThrow(() -> new CategoryNotFoundException("Category is not found with id : " + productDto.getCategory()));
        ProductName productName = productNameRepo.findById(productDto.getProductName())
                .orElseThrow(() -> new ProductNameNotFoundException("Product is not found with id : " + productDto.getProductName()));
        if (brand == null || category == null || productName == null) {
            throw new NullPointerException("Please Enter Valid Brand ID or Category ID or ProductName ID");
        } else {
            String productId = idGeneratorUtility.productIdGenerator();
            Product product = Product.builder().productId(productId).brand(brand).category(category).productName(productName).isActive(productDto.getIsActive())
                    .size(productDto.getSize()).price(productDto.getPrice()).idealFor(productDto.getIdealFor()).vendorAccount(productDto.getVendorAccount())
                    .platformCharge(productDto.getPlatformCharge()).build();
            return productRepository.save(product);
        }
    }

    @Override
    public Product updateProductById(String productId, ProductDto productDto) {
        Optional<Product> product = productRepository.findById(productId);
        if (product.isPresent()) {
            Product existingProduct = product.get();
            existingProduct.setIdealFor(productDto.getIdealFor().equals(Optional.empty()) ? existingProduct.getIdealFor() : productDto.getIdealFor());
            existingProduct.setSize(productDto.getSize().equals(Optional.empty()) ? existingProduct.getSize() : productDto.getSize());
            existingProduct.setPrice(productDto.getPrice().equals(Optional.empty()) ? existingProduct.getPrice() : productDto.getPrice());
            Product updateProductById = productRepository.save(existingProduct);
            return updateProductById;
        } else
            return null;
    }

    @Override
    public void deleteProductById(String productId) {
        if (productId == null) {
            throw new NullPointerException("Enter a Valid Product ID");
        } else {
            Boolean active = productRepository.findById(productId).get().getIsActive();
            if (active) {
                productRepository.deleteById(productId);
            } else {
                productRepository.unDeleteById(productId);
            }
        }
    }

    @Override
    public Product getProductById(String productId) {
        Optional<Product> product = productRepository.findById(productId);
        return product
                .orElseThrow(() -> new ProductNotAvailableException("product is not available with the id " + productId));
    }

    @Override
    public List<Product> getAllProducts() {
        List<Product> products = productRepository.findAllByIsActive(true);
        if (!products.isEmpty()) {
            return products;
        } else
            throw new ProductNotAvailableException("Product List Empty");
    }

    @Override
    public List<OrderDao> getAllOrders() {
        List<OrderDao> orderDaoList = orderRepository.findAll();
        if (!orderDaoList.isEmpty()) {
            return orderDaoList;
        } else
            throw new OrderListEmptyException("Order List Empty");
    }

}
