package com.nisum.splitmerchantpayment.service;

import com.nisum.splitmerchantpayment.entity.OrderedProduct;
import com.nisum.splitmerchantpayment.exception.CustomerNotFoundException;
import com.nisum.splitmerchantpayment.exception.InvalidAmountException;
import com.nisum.splitmerchantpayment.exception.OrderNotFoundException;
import com.nisum.splitmerchantpayment.exception.ProductNotAvailableException;
import com.nisum.splitmerchantpayment.exception.QuantityNotAvailableException;
import com.nisum.splitmerchantpayment.exception.SizeTypeNotAvailableException;
import com.nisum.splitmerchantpayment.model.ProductToCartDto;
import com.nisum.splitmerchantpayment.repository.BaseRepository;
import com.nisum.splitmerchantpayment.repository.CartProductRepository;
import com.nisum.splitmerchantpayment.repository.CustomerRepository;
import com.nisum.splitmerchantpayment.repository.ProductRepository;
import com.nisum.splitmerchantpayment.entity.CartProduct;
import com.nisum.splitmerchantpayment.entity.Customer;
import com.nisum.splitmerchantpayment.entity.OrderDao;
import com.nisum.splitmerchantpayment.entity.Product;
import com.nisum.splitmerchantpayment.entity.Size;
import com.razorpay.Account;
import com.razorpay.Order;
import com.razorpay.RazorpayClient;
import com.razorpay.RazorpayException;
import org.jetbrains.annotations.NotNull;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private BaseRepository baseRepo;

    @Autowired
    private ProductRepository productRepo;

    @Autowired
    private CustomerRepository customerRepo;

    @Autowired
    private CartProductRepository cartProductRepository;

    @Autowired
    private RazorpayClient razorpayClient;

    private static String currency = "INR";

    @Override
    public Product getProductById(String productId) {
        Optional<Product> product = productRepo.findById(productId);
        return product
                .orElseThrow(() -> new ProductNotAvailableException("product is not available with the id " + productId));
    }

    public boolean quantityCheck(Size size, ProductToCartDto productToCartDto) {
        String columnName = productToCartDto.getSize().toLowerCase();
        Field[] fields = Size.class.getDeclaredFields();
        boolean flag = false;
        for (Field field : fields) {
            field.setAccessible(true);
            String fieldName = field.getName();
            if (fieldName.equals(columnName)) {
                Integer requireQuantity = null;
                try {
                    requireQuantity = (int) field.get(size);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException("Quantity not available");
                }
                Integer quantity = productToCartDto.getQuantity();
                flag = requireQuantity >= quantity;
            }
        }
        return flag;
    }

    public Product checkAvailability(ProductToCartDto productToCartDto) {
        Product product = getProductById(productToCartDto.getProductId());
        Size size = product.getSize();
        boolean availabilityFlag = false;
        String sizeType = productToCartDto.getSize().toLowerCase();
        if (sizeType.equals("s") || sizeType.equals("m")
                || sizeType.equals("l") || sizeType.equals("xl") || sizeType.equals("xxl")) {
            availabilityFlag = quantityCheck(size, productToCartDto);
        } else {
            throw new SizeTypeNotAvailableException("Product not available for the size " + sizeType);
        }

        if (availabilityFlag) {
            return product;
        } else
            throw new QuantityNotAvailableException("Quantity not available to the product");
    }

    @Override
    public String addToCart(ProductToCartDto productToCartDto) {
        Optional<Customer> customerOptional = customerRepo.findByPhoneNumber(productToCartDto.getCustomerPhoneNumber());
        customerOptional.orElseThrow(() -> new CustomerNotFoundException("No customer found with the phone number"));
        Product product = checkAvailability(productToCartDto);
        CartProduct cartProduct = CartProduct.builder().productId(product.getProductId())
                .productBrand(product.getBrand().getName())
                .productName(product.getProductName().getName())
                .category(product.getCategory().getName())
                .price(product.getPrice())
                .idealFor(product.getIdealFor())
                .quantity(productToCartDto.getQuantity())
                .size(productToCartDto.getSize())
                .customerPhoneNumber(productToCartDto.getCustomerPhoneNumber())
                .build();
        cartProductRepository.save(cartProduct);
        return "added To cart";
    }

    @Override
    public List<CartProduct> getCartList(String customerPhoneNumber) {
        return cartProductRepository.findAllByCustomerPhoneNumber(customerPhoneNumber);
    }

    @Transactional
    @Override
    public OrderDao placeOrder(String customerPhoneNumber, Double amount) {

        List<Object> transfers = createTransfers(customerPhoneNumber);
        Order order = createTransaction(transfers, amount);

        Customer customer = customerRepo
                .findByPhoneNumber(customerPhoneNumber)
                .orElseThrow(() -> new CustomerNotFoundException("No customer found with the phone number"));

        AtomicReference<Double> bill = new AtomicReference<>(0.00);

        OrderDao orderDao = OrderDao.builder().orderId(order.get("id")).build();

        List<CartProduct> cartProducts = cartProductRepository.findAllByCustomerPhoneNumber(customerPhoneNumber);

        if (!cartProducts.isEmpty()) {
            List<OrderedProduct> orderedProducts = cartProducts.stream().map(product -> {
                baseRepo.updateProduct(product.getSize(), product.getProductId(), product.getQuantity());
                bill.updateAndGet(v -> v + product.getPrice() * product.getQuantity());
                return product;
            }).map(product -> OrderedProduct.builder()
                    .productId(product.getProductId()).productName(product.getProductName())
                    .productBrand(product.getProductBrand())
                    .category(product.getCategory())
                    .idealFor(product.getIdealFor())
                    .size(product.getSize())
                    .price(product.getPrice())
                    .quantity(product.getQuantity())
                    .build()).collect(Collectors.toList());
            orderDao.setOrderedProductList(orderedProducts);
            orderDao.setBill(bill.get());
        }
        customer.getOrderDao().add(orderDao);

        if (!amount.equals(bill.get())) {
            throw new InvalidAmountException("Transaction failed due to less or invalid amount. Please pay " + bill.get());
        }
        Customer customerAddedOrder = customerRepo.save(customer);
        cartProductRepository.deleteAllByCustomerPhoneNumber(customerPhoneNumber);
        return customerAddedOrder
                .getOrderDao()
                .stream()
                .filter(o -> o.getOrderId().equals(orderDao.getOrderId())).findFirst().get();
    }

    public Order createTransaction(List<Object> transfers, Double amount) {
        try {
            JSONObject orderRequest = new JSONObject();
            orderRequest.put("amount", (amount * 100));
            orderRequest.put("currency", currency);
            // orderRequest.put("receipt", "receipt#1");
            orderRequest.put("transfers", transfers);

            Order order = razorpayClient.orders.create(orderRequest);
            return order;
        } catch (RazorpayException e) {
            System.out.println(e.getMessage());
            return null;
        }
    }

    @NotNull
    private List<Object> createTransfers(String customerPhoneNumber) {
        List<CartProduct> cartList = getCartList(customerPhoneNumber);
        List<Object> transfers = new ArrayList<>();
        cartList.stream().forEach(cartProduct -> {
            JSONObject transferParams = new JSONObject();
            Product product = productRepo.findById(cartProduct.getProductId()).get();
            Double platformCharge = ((product.getPlatformCharge() / 100) * product.getPrice() * 100) * cartProduct.getQuantity();
            if (product.getVendorAccount() != null) {
                transferParams.put("account", product.getVendorAccount());
                transferParams.put("amount", product.getPrice().intValue() * cartProduct.getQuantity() * 100 - platformCharge.intValue());
                transferParams.put("currency", "INR");
                JSONObject notes = new JSONObject();
                try {
                    Account vendorAccount = razorpayClient.account.fetch(product.getVendorAccount());
                    notes.put("name", vendorAccount.get("legal_business_name").toString());
                    transfers.add(transferParams);
                } catch (RazorpayException e) {
                    throw new RuntimeException(e);
                }
            }

        });
        return transfers;
    }

    @Override
    public List<OrderDao> getOrdersByCustomer(String customerPhoneNumber) {
        Customer customer = customerRepo.findByPhoneNumber(customerPhoneNumber)
                .orElseThrow(() -> new CustomerNotFoundException("No customer found with the phone number"));
        return customer.getOrderDao();
    }

    @Override
    public OrderDao getOrderByCustomerIdAndOrderId(String customerPhoneNumber, String orderId) {

        return getOrdersByCustomer(customerPhoneNumber)
                .stream()
                .filter(order -> order.getOrderId().equals(orderId))
                .reduce((order, order2) -> order2)
                .orElseThrow(() -> new OrderNotFoundException("Order not found with given oder id : " + orderId));
    }
}